# -*- coding: utf-8 -*-
# Calculation of calibration coefficients for the Hantek DSO2000 oscilloscope generator
# To calibrate the generator, you need to get 2 values. For this:
# 1. Flash an empty calibration file into the oscilloscope (package dso3kb_calibration_restore_000.upk in the attachment).
# 2. Reset the oscilloscope to defaul with the button.
# 3. Start calibration via the " utility " button.
# 4. Set the generator to sine, frequency 1kHz, amplitude 1V.
# 5. Connect the probe of the 1st channel to the generator output.
# 6. Turn on the vertical cursors and set them to the peaks of the sinusoid (see picture).
# 7. Record 2 values ​​AY(positive) and BY(negative) from the top left corner
# 8. Go to C:\Test folder run calc_calibration_upk.py
# 9. Flash the resulting calibration file into the oscilloscope.

import sys
import struct
import os.path
import tarfile
import tempfile
import time
import shutil

def calc_ratio(AY, BY):
    A1 = AY - BY
    B1 = -AY - BY
    A = 1000.00 / A1
    B = B1 / A / 10.00
    return A, B

def calc_hex(A, B):
    hex_A = struct.pack('<d', A)
    hex_B = struct.pack('<d', B)
    return hex_A, hex_B

def make_upk(fname):
    old_dir = os.getcwd()
    workdir = os.path.join(tempfile.gettempdir(), "%f" % time.time())
    os.mkdir(workdir)
    os.chdir(workdir)
    os.mkdir('calibration_restore')
    os.mkdir(os.path.join('calibration_restore', 'package'))

    shutil.copy(os.path.join(old_dir, 'package', 'do_update.sh'), os.path.join('calibration_restore', 'package', 'do_update.sh'))
    shutil.copy(os.path.join(old_dir, fname), os.path.join('calibration_restore', 'package', 'dds_calbration.dat'))
    shutil.copy(os.path.join(old_dir, 'package', 'Manifest.xml'), os.path.join('calibration_restore', 'package', 'Manifest.xml'))
    os.chdir('calibration_restore')
    tar = tarfile.open(os.path.join('..', 'dso3kb.upk.tar'), 'w', format=tarfile.GNU_FORMAT)
    tar.add(os.path.join('.', 'package'))
    tar.close()
    os.chdir('..')
    tar = tarfile.open('dso3kb_calibration_restore.upk.tar.gz', 'w:gz', format=tarfile.GNU_FORMAT)
    tar.add(os.path.join('.', 'dso3kb.upk.tar'))
    tar.close()
    shutil.copy('dso3kb_calibration_restore.upk.tar.gz', old_dir)
    os.chdir(old_dir)
    shutil.rmtree(workdir)
    if sys.platform == 'win32':
        run_code = os.system('.\gpg\gpg.exe --no-autostart --batch --passphrase dso3000c --cipher-algo AES --output dso3kb_calibration_restore.upk -c dso3kb_calibration_restore.upk.tar.gz')
        if run_code == 2:
            run_code = 0
    else:
        run_code = os.system('gpg --batch --passphrase dso3000c --cipher-algo AES --output dso3kb_calibration_restore.upk -c dso3kb_calibration_restore.upk.tar.gz')
    return run_code

if __name__ == '__main__':
    if len(sys.argv) < 3 or len(sys.argv) > 4:
        print('Usage: %s <AY> <BY> [calibration_file]' % sys.argv[0])
        print('Example: %s 584 -600 dds_calbration_001.dat' % sys.argv[0])
        exit()
    try:
        AY = float(sys.argv[1])
        BY = float(sys.argv[2])
    except:
        print('AY and BY must be a number')
        exit()
    if AY < BY:
        print('AY must be > BY')
        exit()
    fname = 'dds_calbration.dat'
    if len(sys.argv) == 4:
        fname = sys.argv[3]
    if os.path.exists(fname):
        print('File "%s" exist. Please use another calibration_file in command line.' % fname)
        exit()
    if os.path.exists('dso3kb_calibration_restore.upk.tar.gz'):
        print('File "dso3kb_calibration_restore.upk.tar.gz" exist. Please remove or rename.')
        exit()
    if os.path.exists('dso3kb_calibration_restore.upk'):
        print('File "dso3kb_calibration_restore.upk" exist. Please remove or rename.')
        exit()
    A, B = calc_ratio(AY, BY)
    hex_A, hex_B = calc_hex(A, B)
    print('A = %f, HEX value: %s' % (A, hex_A.hex()))
    print('B = %f, HEX value: %s' % (B, hex_B.hex()))
    f = open(fname, "wb")
    f.write(bytes([0x10, 00, 0x10, 00, 00, 00, 00, 00]))
    f.write(hex_A)
    f.write(bytes([0x00])*16)
    f.write(hex_B)
    f.close()
    print('Calculation complete. Write file: %s' % fname)
    r = make_upk(fname)
    if r == 0:
        print('File dso3kb_calibration_restore.upk create')
        os.remove('dso3kb_calibration_restore.upk.tar.gz')
        os.remove('dds_calbration.dat')
    
